-- The top module of Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2019/02/13

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity ssd_hw is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  WR_BUF_SEL : in std_logic_vector(1 downto 0);
  WR_BUF_SEL2 : in std_logic_vector(1 downto 0);
  WR_BUF_N : in std_logic;
  RD_BUF_SEL : in std_logic_vector(1 downto 0);
  RD_BUF_ADDR : in std_logic_vector(7 downto 0);
  SsdLineCount : in std_logic_vector(8 downto 0);
  SsdLineCount2 : in std_logic_vector(8 downto 0);
  SsdPixCount : in std_logic_vector(6 downto 0);
  SsdPixCount2 : in std_logic_vector(6 downto 0);
  OutChSel : in std_logic_vector(7 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO256 : in std_logic_vector(9 downto 0);
  CO256_dly2 : in std_logic_vector(9 downto 0);
--		BigLineN : in std_logic_vector(8 downto 0);
		DISPJUDGE : in std_logic;
  ReluWrN : out std_logic;
  ReluWrData : out std_logic_vector(839 downto 0);
  ReluWrAddr : out std_logic_vector(1 downto 0);
  ReluWrSel : out std_logic_vector(1 downto 0);
  LayNextWrN : in std_logic;
  LayNextWrData : in std_logic_vector(2047 downto 0);
  LayNextWrAddr : in std_logic_vector(2 downto 0);
  LayNextWrSel : in std_logic_vector(1 downto 0);
		ReluSum : out std_logic_vector(11 downto 0) );
end ssd_hw;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of ssd_hw is

component conv_conf5
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  OutChSel : in std_logic_vector(7 downto 0);
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO256 : in std_logic_vector(9 downto 0);
  CO256_dly2 : in std_logic_vector(9 downto 0);
  X_RD_DATA : in std_logic_vector(2047 downto 0); 
  ReluAll : out std_logic_vector(839 downto 0); 
  ReluAllWrN : out std_logic; 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end component;

component SRAM_4x2048
Port (
          WRCLK : In std_logic; 
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(1 downto 0);
          RDADDR : In std_logic_vector(1 downto 0);
          WRDATA : In std_logic_vector(2047 downto 0);
          RDDATA : Out std_logic_vector(2047 downto 0) );
end component;

component SRAM_4x840
Port (
          WRCLK : In std_logic; 
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(1 downto 0);
          RDADDR : In std_logic_vector(1 downto 0);
          WRDATA : In std_logic_vector(839 downto 0);
          RDDATA : Out std_logic_vector(839 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal BchData : std_logic_vector(6 downto 0);
signal GchData : std_logic_vector(6 downto 0);
signal RchData : std_logic_vector(6 downto 0);
signal Bch_oft, Bch : std_logic_vector(5 downto 0);
signal Gch_oft, Gch : std_logic_vector(6 downto 0);
signal Rch_oft, Rch : std_logic_vector(5 downto 0);

signal RAM_X1_WR_N_00, RAM_X1_WR_N_01 : std_logic;
signal RAM_X1_WR_N_02, RAM_X1_WR_N_03 : std_logic;
signal RAM_X1_WR_ADDR : std_logic_vector(1 downto 0);
signal RAM_X1_RD_ADDR : std_logic_vector(1 downto 0);
signal RAM_X1_RD_DATA_00, RAM_X1_RD_DATA_01 : std_logic_vector(2047 downto 0);
signal RAM_X1_RD_DATA_02, RAM_X1_RD_DATA_03 : std_logic_vector(2047 downto 0);
signal RAM_X1_RD_DATA_03_padding : std_logic_vector(2047 downto 0);
signal RAM_X1_RD_DATA : std_logic_vector(2047 downto 0);
signal RAM_X1_RD_SEL : std_logic_vector(1 downto 0);

signal RAM_X2_WR_N_00, RAM_X2_WR_N_01 : std_logic;
signal RAM_X2_WR_ADDR : std_logic_vector(1 downto 0);
signal RAM_X2_RD_ADDR : std_logic_vector(1 downto 0);
signal RAM_X2_WR_DATA : std_logic_vector(839 downto 0);

signal ReluAll : std_logic_vector(839 downto 0);
signal ReluAllWrN : std_logic;
signal ReluWrN_sig : std_logic;
signal ReluWrN_tmp : std_logic;

begin

---------------------------------------------------------
  ReluWrN <= ReluWrN_sig;
  ReluWrSel <= WR_BUF_SEL;
  ReluWrData <= RAM_X2_WR_DATA;
  ReluWrAddr <= RAM_X2_WR_ADDR;

---------------------------------------------------------
RAM_X1_00 : SRAM_4x2048 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X1_WR_N_00, 
   WRADDR	=> RAM_X1_WR_ADDR, 
   RDADDR	=> RAM_X1_RD_ADDR, 
   WRDATA	=> LayNextWrData, -- RAM_X1_WR_DATA, 
   RDDATA	=> RAM_X1_RD_DATA_00);

RAM_X1_WR_N_00 <= '0' when LayNextWrN = '0' and LayNextWrSel = 0 else '1';

---------------------------------------------------------
RAM_X1_01 : SRAM_4x2048 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X1_WR_N_01, 
   WRADDR	=> RAM_X1_WR_ADDR, 
   RDADDR	=> RAM_X1_RD_ADDR, 
   WRDATA	=> LayNextWrData, -- RAM_X1_WR_DATA, 
   RDDATA	=> RAM_X1_RD_DATA_01);

RAM_X1_WR_N_01 <= '0' when LayNextWrN = '0' and LayNextWrSel = 1 else '1';

---------------------------------------------------------
RAM_X1_02 : SRAM_4x2048 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X1_WR_N_02, 
   WRADDR	=> RAM_X1_WR_ADDR, 
   RDADDR	=> RAM_X1_RD_ADDR, 
   WRDATA	=> LayNextWrData, -- RAM_X1_WR_DATA, 
   RDDATA	=> RAM_X1_RD_DATA_02);

RAM_X1_WR_N_02 <= '0' when LayNextWrN = '0' and LayNextWrSel = 2 else '1';

---------------------------------------------------------
RAM_X1_03 : SRAM_4x2048 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X1_WR_N_03, 
   WRADDR	=> RAM_X1_WR_ADDR, 
   RDADDR	=> RAM_X1_RD_ADDR, 
   WRDATA	=> LayNextWrData, -- RAM_X1_WR_DATA, 
   RDDATA	=> RAM_X1_RD_DATA_03);

RAM_X1_WR_N_03 <= '0' when LayNextWrN = '0' and LayNextWrSel = 3 else '1';

---------------------------------------------------------
RAM_X1_WR_ADDR <= LayNextWrAddr(1 downto 0);
RAM_X1_RD_ADDR <= RD_BUF_ADDR(1 downto 0);

RAM_X1_RD_SEL <= RD_BUF_SEL - 1; -- RAM_X1 is written from '0' -- from -3 because of delay one more line 

RAM_X1_RD_DATA <= RAM_X1_RD_DATA_00 when RAM_X1_RD_SEL = 0 else 
		RAM_X1_RD_DATA_01 when RAM_X1_RD_SEL = 1 else
		RAM_X1_RD_DATA_02 when RAM_X1_RD_SEL = 2 else
		RAM_X1_RD_DATA_03_padding;

RAM_X1_RD_DATA_03_padding <= (others => '0') when SsdLineCount = 3 or SsdLineCount = 5 else RAM_X1_RD_DATA_03; -- for conf5

conv_conf5_inst : conv_conf5 port map (
          CLK => CLK,
          RST_N => RST_N,
          OutChSel => OutChSel,
          CO300 => SsdLineCount,
          CO9 => CO9,
          CO256 => CO256,
          CO256_dly2 => CO256_dly2,
          X_RD_DATA => RAM_X1_RD_DATA,
          ReluAll => ReluAll,
          ReluAllWrN => ReluAllWrN,
          ReluSum => open);

RAM_X2_WR_ADDR <= SsdPixCount(1 downto 0); -- for conf5
---RAM_X2_WR_ADDR <= SsdPixCount(3 downto 0);
---RAM_X2_WR_ADDR <= Pooladdr; -- layer13 has poolings
RAM_X2_WR_DATA <= ReluAll;
---RAM_X2_WR_DATA <= Poolout; -- layer13 has poolings

ReluWrN_tmp <= '0' when ReluAllWrN = '0'  
		and (SsdLineCount2 >= 3 and SsdLineCount2 <= 5)  -- for conf5
		else '1'; 

ReluWrN_sig <= '0' when ReluWrN_tmp = '0' and SsdPixCount >= 1 and SsdPixCount <= 3 else '1'; -- for conf5

--process( RST_N, CLK )  
--begin 
--   if( RST_N = '0' ) then
--     ReluWrN_sig <= '0';
--   elsif( CLK'event and CLK = '1' ) then
--     ReluWrN_sig <= ReluWrN_tmp; -- to catch addres '38'(half period)
--  end if;
--end process;

---------------------------------------------------------
RAM_X2_00 : SRAM_4x840 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X2_WR_N_00, 
   WRADDR	=> RAM_X2_WR_ADDR, 
   RDADDR	=> RAM_X2_RD_ADDR, 
   WRDATA	=> RAM_X2_WR_DATA, 
   RDDATA	=> open);

RAM_X2_WR_N_00 <= '0' when ReluWrN_sig = '0' and WR_BUF_SEL(0) = '0'   -- sel for ex2
		else '1'; 

---------------------------------------------------------
RAM_X2_01 : SRAM_4x840 port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> RAM_X2_WR_N_01, 
   WRADDR	=> RAM_X2_WR_ADDR, 
   RDADDR	=> RAM_X2_RD_ADDR, 
   WRDATA	=> RAM_X2_WR_DATA, 
   RDDATA	=> open);

RAM_X2_WR_N_01 <= '0' when ReluWrN_sig = '0' and WR_BUF_SEL(0) = '1'  -- sel for ex2
		else '1'; 


end RTL;

